from terragit.utilsFunctions import *
from terragit.terragrunt import *


class AddUserGitlabRepo:
    def __init__(self, gitlab_token, git_url):
        self.gitlab_token = gitlab_token
        self.git_url = git_url
        self.gitlab_functions = GitlabFunctions(gitlab_token, git_url)
        self.bcolor = bcolors()
        self.utils_functions = UtilsFunctions(gitlab_token, git_url)

    def add_user(self, project_id, grp_id, gitlab_username, access_level, exist, keybase):
        headers = {'PRIVATE-TOKEN': self.gitlab_token}
        if "user does not exist in gitlab group" in exist:
            if access_level.lower() == "guest":
                level = 10
            elif access_level.lower() == "reporter":
                level = 20
            elif access_level.lower() == "developer":
                level = 30
            elif access_level.lower() == "maintainer":
                level = 40
            elif access_level.lower() == "owner":
                level = 50
            url = self.git_url + "/api/v4/users?username=" + str(gitlab_username)
            user = requests.get(url, headers=headers).json()
            if grp_id is not None:
                url = self.git_url + "/api/v4/groups/" + str(grp_id) + "/members?user_id=" + str(
                    user[0]['id']) + "&access_level=" + str(level)
                add = requests.post(url, headers=headers)
        if "IAM user not found" in exist:
            self.utils_functions.apply_service(project_id, self.gitlab_token, "addUser", "create_user", "master",
                                               gitlab_username, keybase)

    def check_user_existance(self, project_id, grp_id, gitlab_username):
        headers = {'PRIVATE-TOKEN': self.gitlab_token}
        exist = ""
        url = self.git_url + "/api/v4/users?username=" + str(gitlab_username)
        user = requests.get(url, headers=headers).json()
        try:
            url = self.git_url + "/api/v4/groups/" + str(grp_id) + "/members/all?user_ids=" + str(user[0]['id'])
            exist = requests.get(url, headers=headers).json()
            # print("exist", exist)
            if len(exist) > 0:
                exist = "user exist in gitlab group & "
            else:
                exist = "user does not exist in gitlab group & "
        except:
            exist = "problem occured while checking user existance in gitlab "
            print(bcolors.FAIL, exist)
            return exist
        res = self.utils_functions.check_content_existance_in_service(project_id,
                                                                      'https://git@gitlab.com/commons-acp/terraform/aws/user.git',
                                                                      'name="' + gitlab_username + '"')
        if "content_found" in res:
            print(bcolors.OKGREEN, exist + "IAM user found")
            return exist + "IAM user found"
        else:
            print(bcolors.OKGREEN, exist + "IAM user found")
            return exist + "IAM user not found"

    def existance_gitlab(self, project_id, group_id, gitlab_username):

        headers = {'PRIVATE-TOKEN': self.gitlab_token}
        url = self.git_url + "/api/v4/users?username=" + str(gitlab_username)
        user = requests.get(url, headers=headers).json()
        if len(user) != 0:

            if project_id is not None:
                url = self.git_url + "/api/v4/projects/" + str(project_id) + "/members/all/" + str(user[0]['id'])
                exist = requests.get(url, headers=headers).json()
                try:
                    if 'message' in exist:
                        exist = "user does not exist in gitlab project & "
                    else:
                        exist = "user exist in gitlab project & "
                    return exist, user[0]['id']
                except:
                    exist = "problem occured while checking user existance in gitlab "
                    print(bcolors.FAIL, exist)
                    return exist, user[0]['id']
            if group_id is not None:
                url = self.git_url + "/api/v4/groups/" + str(group_id) + "/members/all/" + str(user[0]['id'])
                exist = requests.get(url, headers=headers).json()
                try:
                    if 'message' in exist:
                        exist = "user does not exist in gitlab project & "
                    else:
                        exist = "user exist in gitlab project & "
                    return exist, user[0]['id']
                except:
                    exist = "problem occured while checking user existance in gitlab "
                    print(bcolors.FAIL, exist)
                    return exist, user[0]['id']
        else:
            print(self.bcolor.WARNING, " no user found in gitlab using this username")

    def add_user_repository(self, project_id, group_id, gitlab_username, access_level, exist, keybase):

        exist_infra = int
        if exist is not None:
            if "user does not exist in gitlab project & " in exist[0]:
                if project_id is not None:
                    current_group = self.gitlab_functions.current_group_project(project_id)
                    exist_infra = self.gitlab_functions.check_project_infra_exist(current_group['id'])
                if group_id is not None:
                    exist_infra = self.gitlab_functions.check_project_infra_exist(group_id)

                if exist_infra is None:
                    print(bcolors.FAIL + " there is no infra project in this group")
                else:
                    self.gitlab_functions.create_branch(exist_infra, gitlab_username + "-patch" + str(exist_infra))
                    if project_id is not None:
                        Data = {
                            'branch': gitlab_username + "-patch" + str(exist_infra),
                            'commit_message': 'adding membership for a user ' + gitlab_username + ' to project',
                            'actions': [
                                {
                                    'action': 'create',
                                    'file_path': 'live/git/users/'+gitlab_username+'/main.tf',
                                    'content':
                                        'terraform  {\n' +
                                        '  required_providers {\n' +
                                        '    gitlab = {\n' +
                                        '       source  = "gitlabhq/gitlab"\n' +
                                        '       version = ">= 3.4"\n' +
                                        '     }\n' +
                                        '  }\n' +
                                        '}\n'
                                        '\n'
                                        'resource "gitlab_project_membership" "membership" {\n' +
                                        '  project_id = "' + str(project_id) + '"\n' +
                                        '  user_id = "' + str(exist[1]) + '"\n' +
                                        '  access_level = "' + access_level.lower() + '"\n' +
                                        '}\n'

                                },
                                {
                                    'action': 'create',
                                    'file_path': 'live/git/users/'+gitlab_username+'/terragrunt.hcl',
                                    'content':
                                        'include {\n' +
                                        '  path = find_in_parent_folders()\n' +
                                        '}\n'
                                },

                            ]
                        }
                    if group_id is not None:
                        Data = {
                            'branch': gitlab_username + "-patch" + str(exist_infra),
                            'commit_message': 'adding membership for a user ' + gitlab_username + ' to group',
                            'actions': [
                                {
                                    'action': 'create',
                                    'file_path': 'live/git/users/'+gitlab_username+'/main.tf',
                                    'content':
                                        'terraform  {\n' +
                                        '  required_providers {\n' +
                                        '    gitlab = {\n' +
                                        '       source  = "gitlabhq/gitlab"\n' +
                                        '       version = ">= 3.4"\n' +
                                        '     }\n' +
                                        '  }\n' +
                                        '}\n'
                                        '\n'
                                        'resource "gitlab_group_membership" "membership" {\n' +
                                        '  group_id = "' + str(group_id) + '"\n' +
                                        '  user_id = "' + str(exist[1]) + '"\n' +
                                        '  access_level = "' + access_level.lower() + '"\n' +
                                        '}\n'

                                },
                                {
                                    'action': 'create',
                                    'file_path': 'live/git/users/'+gitlab_username+'/terragrunt.hcl',
                                    'content':
                                        'include {\n' +
                                        '  path = find_in_parent_folders()\n' +
                                        '}\n'
                                },

                            ]
                        }
                    url = self.git_url + "/api/v4/projects/" + str(exist_infra) + "/repository/commits"
                    add = requests.post(url, json=Data, headers={'PRIVATE-TOKEN': self.gitlab_token})
                    mr_request_creation = self.gitlab_functions.merge_request(exist_infra, gitlab_username + "-patch" + str(exist_infra), "master")
                    while mr_request_creation.json()['pipeline'] is None:
                        mr = self.gitlab_functions.get_all_merge_request_by_project_id(exist_infra, mr_request_creation.json()['iid'])
                        if mr['pipeline'] is not None:
                            accept_mr = self.gitlab_functions.accept_merge_request(exist_infra, mr['iid'])
                            if mr['state'] == 'merged':
                                break






import os
import subprocess
class ProjectStatus:
    def __init__(self, colors):
        self.colors = colors

    def gitstatus(self, dirs):
        for dirpath, dirnames, filenames in os.walk(dirs):
            if self.is_git_workdir_or_repo(dirpath, dirnames, filenames):
                verif = 0
                cmd = "cd " + dirpath + " && git status"
                popen = subprocess.Popen(cmd, stdin=subprocess.PIPE, stdout=subprocess.PIPE, shell=True,
                                         encoding='utf8')
                lines = popen.communicate(input='\n')[0].split("\n")
                for line in lines:
                    if ("nothing to commit, working tree clean" in line):
                        print(self.colors.OKGREEN + "nothing to commit in " + dirpath + " project")
                        verif = 1
                        break
                if verif == 0:
                    print(self.colors.WARNING + "you have uncommitted changes " + dirpath + " project")

    def is_git_workdir_or_repo(self, dirpath, dirnames, filenames):
        if (".git" in dirnames or ".git" in filenames) and (".terraform" not in dirpath):
            return 1
        return 0



import datetime
import re
import base64
import time

import terragit.terragrunt as terragrunt
from terragit.gitlabFunctions import *
from terragit.keybaseFunctions import *




class UtilsFunctions:
    def __init__(self, gitlab_token, git_url):
        self.gitlab_functions = GitlabFunctions(gitlab_token, git_url)
        self.git_url = git_url
        self.gitlab_token = gitlab_token
        self.bcolor = terragrunt.bcolors
        self.keybase_function = KeybaseFunctions()


    def track_pipeline(self, project_id, mr_id, i):
        print('waiting for service to be created')
        print(self.bcolor.OKGREEN, "waiting for service to be created ")
        res = self.gitlab_functions.get_last_mr_commit_id(project_id, mr_id)
        if i == 1:
            time.sleep(5)
        commit = self.gitlab_functions.get_single_commit(project_id, res[0]['id'])
        jobs = self.gitlab_functions.get_pipeline_jobs(project_id, commit['last_pipeline']['id'])
        for job in jobs:
            if job['status'] == 'failed':
                return "failed " + str(job['id'])
            elif job['name'] == 'apply-all':
                if job['status'] == 'success':
                    return str(job['id'])
            elif job['stage'] == 'merge' and job['status'] == 'success':
                self.gitlab_functions.accept_merge_request(project_id, mr_id)
                return 'false'
        return 'false'

    def apply_service(self, project_id, gitlab_token, service_name, param, main_branch="master",
                      aws_username="", keybase=""):

        date = datetime.datetime.now()
        date_replaced = re.sub(r' ', '-', str(date))
        date_replaced = re.sub(r'\.', '-', str(date_replaced))
        date_replaced = re.sub(r':', '-', str(date_replaced))
        # print(date_replaced)
        self.gitlab_functions.create_branch(project_id, service_name + "-" + date_replaced)
        if param == "create_user":
            aws_username_replaced = re.sub(r'\.', '%2E', aws_username)
            self.gitlab_functions.create_user(service_name + '-' + str(date_replaced),
                                              'live/aws/global/iam/users/' + str(aws_username_replaced),
                                              str(aws_username), str(keybase), str(project_id))
        res = self.gitlab_functions.merge_request(project_id, service_name + "-" + str(date_replaced), main_branch)
        res = res.json()
        mr_id = str(res['iid'])
        job_id = 'false'
        i = 1
        while 'false' in job_id:
            time.sleep(5)
            job_id = self.track_pipeline(project_id, mr_id, i)
            i = i + 1
        # print("job_id", job_id)
        trace = self.gitlab_functions.get_job_trace(project_id, job_id)
        # print("trace", trace)
        if ('false' not in job_id) and ('failed' not in job_id):

            start = trace.rindex('Outputs:')
            end = trace.index('Uploading artifacts')
            outputs = trace[start:end]
            print(outputs)
            return outputs
        elif 'false' not in job_id:
            if ("Error:" in trace) and ("COULDNT PROCESS" in trace):
                error = trace[trace.index('Outputs:'):trace.index("COULDNT PROCESS")]
                print("error ", error)
            else:
                print("error configuration is not valid")

            return "error"

    def check_content_existance_in_service(self, project_id, service_template, content):
        page = 1
        per_page = 20
        while per_page == 20:
            services = self.gitlab_functions.get_services(project_id, page)
            page = page + 1
            per_page = len(services)
            for service in services:
               try:
                   if "main.tf" in service['path']:
                       path_replaced = re.sub(r'/', '%2F', service['path'])
                       path_replaced = re.sub(r'\.', '%2E', path_replaced)
                       res = self.gitlab_functions.get_file_content(project_id, path_replaced)
                       service_content = str(base64.b64decode(res['content'])).replace(" ", "")
                       if (service_template in service_content) and (
                           content in service_content):
                           return "content_found & service_path:" + service['path']
               except:
                   return service['message']
        return "content not found"

    def check_content_existance_in_serviceGr(self, group_id, project_id, service_template, content):
        if project_id!=None :
            return self.check_content_existance_in_service(project_id, service_template, content)
        else:
            pr= self.gitlab_functions.get_projects_of_group(group_id,1,10)
            for i in range(len(pr)):
                print("project of group", pr[i]['id'])
                if pr[i]['name'] == "infra":
                    return self.check_content_existance_in_service(pr[i]['id'], service_template, content)
                else:
                    continue

    def help(self):
        colors = terragrunt.bcolors()
        print(colors.OKGREEN + "Terragit functions:")
        print(colors.OKBLUE + "1-terragit config \n2-terragit {changes, validate, plan, apply, output} \n" +
              "3-terragit docs \n4-terragit clone \n5-terragit adduser \n6-terragit list \n"
              "7-terragit currentProject \n8-terragit switch \n9-terragit clean \n"

              )
        print(colors.OKBLUE + "")
        print()

    def add_aws_account(self, group_id, gitlab_username):

        project_infra_id = self.gitlab_functions.check_project_infra_exist(group_id)
        headers = {'PRIVATE-TOKEN': self.gitlab_token}
        url = self.git_url + "/api/v4/users?username=" + str(gitlab_username)
        user = requests.get(url, headers=headers).json()
        action = ""

        if len(user) == 0:
            print(self.bcolor.FAIL + "this username does not exist please retry")

        else:
            tf_state ={}
            list = []
            folders = self.gitlab_functions.check_with_user_folder(project_infra_id)

            if len(folders) == 0:
                print(self.bcolor.FAIL + " the path doesn't match with the  preconfigured one" + self.bcolor.ENDC)

            else:
                for i in range(0, len(folders)):
                    list.append(folders[i]['name'])

            if "." in gitlab_username:
                tfstate_name = str(gitlab_username).replace(".", "-")
            else:
                tfstate_name = gitlab_username
            tf_state = self.gitlab_functions.get_user_tfstate(project_infra_id, tfstate_name)

            if gitlab_username in list and (len(tf_state['outputs']) != 0 or 'message' in tf_state):
                print(self.bcolor.OKGREEN, "IAM user found in this account")
                value = input(self.bcolor.OKBLUE + "would you like to have your encrypted credentials "
                                                           " ? yes/no  ")

                if value == "no":
                    return
                else:

                    print(self.bcolor.OKGREEN, "PASSWORD : ", self.bcolor.WARNING
                                    + str(tf_state['outputs']['encrypted_password']['value']))
                    print(self.bcolor.OKGREEN, "AWS_ACCESS_KEY_ID : ", self.bcolor.WARNING
                                    + str(tf_state['outputs']['access_key_id']['value']))
                    print(self.bcolor.OKGREEN, "AWS_SECRET_ACCESS_KEY : ", self.bcolor.WARNING
                                    + str(tf_state['outputs']['encrypted_secret_access_key']['value']))
                    print(self.bcolor.OKGREEN, "USER_NAME : ", self.bcolor.WARNING
                                    + str(tf_state['outputs']['user_name']['value']))
            else:
                print(self.bcolor.OKGREEN, "IAM user not found in this account")
                value = input(self.bcolor.OKBLUE + "would you like to create an account? "
                                                   " ? yes/no  ")
                if value == "no":
                    return
                else:
                    value = input(self.bcolor.OKBLUE + " please provide your keybase: "
                                                   "")
                    check_keybase = self.keybase_function.check_keybase_username_existance(value)
                    if check_keybase['them'] == [None]:
                        print(self.bcolor.FAIL, "Keybase not found , please check the provided name")
                    else:

                        self.gitlab_functions.create_branch(project_infra_id, gitlab_username + "-patch" + str(project_infra_id))
                        if len(tf_state['outputs']) == 0:
                            action = "delete"
                            self.gitlab_functions.create_user(str(gitlab_username) + "-patch" + str(project_infra_id),
                                                          'live/aws/global/iam/users/'+str(gitlab_username),
                                                           str(gitlab_username), str(value), str(project_infra_id), action)
                        action = "create"
                        self.gitlab_functions.create_user(str(gitlab_username) + "-patch" + str(project_infra_id),
                                                          'live/aws/global/iam/users/'+str(gitlab_username),
                                                           str(gitlab_username), str(value), str(project_infra_id), action)
                        mr_request_creation = self.gitlab_functions.merge_request(project_infra_id, gitlab_username + "-patch" + str(project_infra_id), "master")
                        job_id = 'false'
                        i = 1
                        while 'false' in job_id:
                            time.sleep(5)
                            job_id = self.track_pipeline(project_infra_id, mr_request_creation.json()['iid'], i)
                            i = i + 1
                        tfstate_created = self.gitlab_functions.get_user_tfstate(project_infra_id, tfstate_name)
                        print(self.bcolor.OKGREEN, "PASSWORD : ", self.bcolor.WARNING
                                + str(tfstate_created['outputs']['encrypted_password']['value']))
                        print(self.bcolor.OKGREEN, "AWS_ACCESS_KEY_ID : ", self.bcolor.WARNING
                                + str(tfstate_created['outputs']['access_key_id']['value']))
                        print(self.bcolor.OKGREEN, "AWS_SECRET_ACCESS_KEY : ", self.bcolor.WARNING
                                + str(tfstate_created['outputs']['encrypted_secret_access_key']['value']))
                        print(self.bcolor.OKGREEN, "USER_NAME : ", self.bcolor.WARNING
                                + str(tfstate_created['outputs']['user_name']['value']))







# class setInterval:
#     def __init__(self, interval, action, project_id, mr_id,gitlab_token):
#         print('setInterval')
#         self.interval = interval
#         self.action = action
#         self.project_id = project_id
#         self.mr_id = mr_id
#         self.gitlab_token = gitlab_token
#         self.stopEvent = threading.Event()
#         thread = threading.Thread(target=self.__setInterval)
#         thread.start()
#
#     async def __setInterval(self):
#
#         print('__setInterval')
#         nextTime = time.time() + self.interval
#         while not self.stopEvent.wait(nextTime - time.time()):
#             print('while')
#             nextTime += self.interval
#             utils_functions = UtilsFunctions(self.gitlab_token, "https://gitlab.com")
#             job_id = utils_functions.track_pipeline(self.project_id, self.mr_id)
#             if 'false' not in job_id:
#                 return job_id
#
#     def cancel(self):
#         self.stopEvent.set()



import gitlab
import os
import sys

class terrareport:
    def __init__(self,folder,git_url,gitlab_token):
        self.git_url =git_url
        self.gitlab_token =gitlab_token
        self.f ="None"
        self.folder =folder
        self.subgrouplist =[]
        self.git_url =git_url

        self.gl = gitlab.Gitlab(git_url,private_token = gitlab_token)

        if not os.path.isdir(os.getcwd()+self.folder+"/reports/modules"):
            os.makedirs(os.getcwd()+self.folder+"/reports/modules")
        self.f = open(os.getcwd()+self.folder+'/reports/modules/README.md', 'w')
        sys.stdout = self.f
        print( "group-name | module-name | tag |")
        print("--- | --- | --- |")

    def myMax(self,liste):
        max = liste[0]
        for x in liste:
            if x > max :
                max = x
        return max
    def getSubgroupList(self,group):

        group = self.gl.groups.get(group)
        self.subgrouplist = group.subgroups.list()
        return  self.subgrouplist
    def getModule(self, subgrouplist,group):
        groupob = self.gl.groups.get(group)
        for subgroupe in subgrouplist:
            sub=self.gl.groups.get(subgroupe.id).subgroups.list()

            if sub:
                self.getModule(sub)

            projects = self.gl.groups.get(subgroupe.id).projects.list()
            for project in projects:
                tags = self.gl.projects.get(project.id).tags.list()
                taglist =[]
                for tag in tags:
                    tagname= tag.name
                    taglist.append(tagname)
                if not  taglist :
                    print(str(subgroupe.web_url[(subgroupe.web_url.index(groupob.name)):len(subgroupe.web_url)])+ "|" +project.name +" | no tags")

                else :
                    print(str(subgroupe.web_url[(subgroupe.web_url.index(groupob.name)):len(subgroupe.web_url)])+ "|" +project.name +  "|" +self.myMax(taglist))
        if not self.subgrouplist:
            groupob= self.gl.groups.get(group)
            projects = groupob.projects.list()
            for project in projects:
                tags = self.gl.projects.get(project.id).tags.list()
                taglist =[]
                for tag in tags:
                    tagname= tag.name
                    taglist.append(tagname)
                if not  taglist :
                    print(str(groupob.web_url[(groupob.web_url.index(groupob.name)):len(groupob.web_url)])+ "|" +project.name +" | no tags")

                else :
                    print(str(groupob.web_url[(groupob.web_url.index(groupob.name)):len(groupob.web_url)])+ "|" +project.name +  "|" +self.myMax(taglist))



from os.path import exists
import json
import os
from terragit.utilsFunctions import *
import subprocess
import terragit.keybaseFunctions as keybaseFunctions
import terragit.clone as clone
import terragit.terragrunt as terragrunt
import terragit.utilsFunctions as utils
import terragit.gitlabFunctions as GitlabFunctions


class TerraConf:
    def __init__(self):
        self.bcolor = terragrunt.bcolors

    def init_file(self):
        conf = {}
        for value in ['gitlab_token', 'keybase', 'gitlab_username']:
            if value == "keybase":
                keybase_api = keybaseFunctions.KeybaseFunctions()
                verif = "false"
                while verif == "false":
                    val = input("Enter your " + value + ": ")
                    check = keybase_api.check_keybase_username_existance(str(val))
                    if check['them'][0] is None:
                        print(self.bcolor.FAIL + "this keybase username does not exist please retry")
                    else:
                        verif = "true"
            else:
                val = input("Enter your " + value + ": ")
            conf[value] = val
        exist = exists(os.path.expanduser(os.path.join("~/.terragit")))

        if not exist:

            file = open(os.path.expanduser(os.path.join("~/.terragit")), mode='w')
            file.write(
                '{ "gitlab_token": "' + conf['gitlab_token'] + '",\n' +
                '"keybase": "' + conf['keybase'] + '",\n' +
                '"gitlab_user": "' + conf['gitlab_username'] + '",\n' +
                '"projects": [] }\n '
            )
        else:
            file = open(os.path.expanduser(os.path.join("~/.terragit")))
            content = json.loads(file.read())
            file.close()
            file = open(os.path.expanduser(os.path.join("~/.terragit")), mode='w')
            content['gitlab_token'] = conf['gitlab_token']
            content['keybase'] = conf['keybase']
            content['gitlab_user'] = conf['gitlab_username']
            json.dump(content, file, indent=3)
        file.close()
        print(self.bcolor.OKGREEN, " Terragit has been successfully initiated")


    def get_file_content(self):
        file = open(os.path.expanduser(os.path.join("~/.terragit")))

        json_object = json.loads(file.read())

        file.close()
        return json_object

    def list_projects(self, json_object):
        print(self.bcolor.OKGREEN, "your projects", json_object["projects"])

    def verif_file_and_credentials_existence(self):
        if exists(os.path.expanduser(os.path.join("~/.terragit"))):
            file = open(os.path.expanduser(os.path.join("~/.terragit")))

            json_object = json.loads(file.read())

            file.close()

            if (len(json_object["gitlab_token"]) > 0) and (len(json_object["gitlab_user"]) > 0) and (
                    len(json_object["keybase"]) > 0):
                return json_object
        return 'false'

    def get_selected_project(self, content):
        for grp_name in list(content.keys())[4:]:
            if content[grp_name]['selected'] == 'true':
                print(self.bcolor.OKGREEN, " your Current selected project is ", content[grp_name])
                return content[grp_name]
        print(self.bcolor.WARNING, " no project is selected")

    def clone_and_switch(self, content, group_id, group_name, pwd):

        #

        colors = terragrunt.bcolors()

        gitlab_url = "https://gitlab.com"

        verif = self.verif_file_and_credentials_existence()
        print(colors.OKCYAN, "Cloning ...")
        clonn = clone.Clone(verif["gitlab_token"], gitlab_url, pwd)
        clonn.clone_projects(None, group_id)
        print(colors.OKCYAN, "Checking the existance of IAM user ...")
        utilsFunctions = utils.UtilsFunctions(verif["gitlab_token"], gitlab_url)
        gt = GitlabFunctions.GitlabFunctions(verif["gitlab_token"], gitlab_url)
        service_path = utilsFunctions.check_content_existance_in_serviceGr(group_id, None,
                                                                           'https://git@gitlab.com/commons-acp/terraform/aws/user.git',
                                                                           'name="' + verif["gitlab_user"] + '"')

        if "content_found" in service_path:
            print(colors.OKGREEN, "IAM user found!")

            self.add_credentials_to_group(gitlab_url, verif["gitlab_token"], group_id,
                                          None, pwd, "/" + service_path[29:service_path.rindex("/")], "true")
        else:
            print("IAM user not found")
            self.add_credentials_to_group(gitlab_url, verif["gitlab_token"], group_id,
                                          None, pwd, "", "false")

        self.switch_project_existant(content, group_id, group_name)

    def switch_project(self, content, group_id, group_name):
        if self.check_project_existance(content, group_id, group_name):
            print("project is already cloned!!")
            self.switch_project_existant(content, group_id, group_name)
        else:
            print("we are going to clone the project for you")
            pwd = os.popen('pwd').read().rstrip()
            pwd = pwd + "/"

            self.clone_and_switch(content, group_id, group_name, pwd)

    def check_project_existance(self, content, group_id, group_name):
        exist = False
        for grp_name in list(content.keys())[4:]:
            if group_name == grp_name:
                if content[group_name]['group_id'] == group_id:
                    exist = True
                    return exist
        return exist

    def switch_project_existant(self, content1, group_id, group_name):
        print("switch_project_existant")
        switched = True
        content = self.get_file_content()

        for grp_name in list(content.keys())[4:]:
            if group_name == grp_name:
                if content[group_name]['group_id'] == group_id:
                    content[group_name]['selected'] = 'true'
                    if content[group_name]['aws_credentials']['secret_access_key'] != None:

                        cmd4 = "export GROUP_ID=" + group_id +"\n"
                        cmd3 = "export GITLAB_USER_NAME=" + content["gitlab_user"] + " \n" + " export gitlab_token=" + \
                           content["gitlab_token"]
                        cmd2 = "export AWS_SECRET_ACCESS_KEY=" + content[group_name]['aws_credentials']['secret_access_key']
                        cmd1 = " \n export AWS_ACCESS_KEY_ID=" + content[group_name]['aws_credentials'][
                            'access_key'] + " \n " + cmd2 + " \n " + cmd3 + " \n " + cmd4

                        f  = open(os.path.expanduser(os.path.join("~/.terragit_profile")), "w")
                        f.write(cmd1)
                        f.close()
                        cmd= "aws --version"
                        child = subprocess.Popen(cmd, stdin=subprocess.PIPE, stdout=subprocess.PIPE, shell=True,
                                                 encoding='utf8')
                        streamdata = child.communicate()[0]
                        print("streamdata",streamdata)
                        rc = child.returncode
                        print("rc",rc)
                        if(rc >= 0):
                            cred=" echo you have no aws access_key to add for this profile/group/project "
                            print("aws-cli in installed")
                            if len(content[group_name]['aws_credentials']['access_key']) > 1:

                                aws_cmd= "aws configure set aws_access_key_id  " \
                                         ""+content[group_name]['aws_credentials']['access_key'] + " --profile "+ group_name

                                cred= aws_cmd
                            if len(content[group_name]['aws_credentials']['secret_access_key']) > 1:
                                aws_cmd1= "aws configure set aws_secret_access_key " + content[group_name]['aws_credentials']['secret_access_key']+" --profile "+ group_name
                                cred= cred+" && " + aws_cmd1


                            subprocess.Popen(cred, stdin=subprocess.PIPE, stdout=subprocess.PIPE, shell=True,
                                             encoding='utf8')
                        else:
                            print("please install aws-cli first")
                    switched = 'true'
                    print("Please run source ~/.terragit_profile")

            elif content[grp_name]['selected'] == 'true':
                content[grp_name]['selected'] = 'false'
        if not switched:
            print(self.bcolor.WARNING, " Error retrieving project")
        else:
            with open(os.path.expanduser(os.path.join("~/.terragit")), 'w') as f:
                json.dump(content, f, indent=3)
            print(self.bcolor.OKGREEN, " Switch to project " + group_name + " succeded")

    def add_credentials_to_group(self, git_url, gitlab_token, group_id, project_id, path1, service_path, exist):
        gitlab_functions1 = GitlabFunctions.GitlabFunctions(gitlab_token, git_url)
        if project_id == None:
            prj = gitlab_functions1.get_projects_of_group(group_id, 1, 10)
            for i in prj:
                prj_path = i['web_url']
                group_name = gitlab_functions1.get_group_by_id(group_id)['name']
                access_key_id = ""
                decrypted_secret_access_key = ""
                path = path1
                project_path = path
                if exist == "true":
                    print(self.bcolor.OKCYAN, "Getting & Decrypting credentials ...")
                    path = path + prj_path[len(git_url) + 1:len(prj_path)] + service_path
                    cmd = "cd " + path + " && export gitlab_token=" + gitlab_token + " && terragrunt output"
                    popen = subprocess.Popen(cmd, stdin=subprocess.PIPE, stdout=subprocess.PIPE, shell=True,
                                             encoding='utf8')
                    lines = popen.communicate(input='\n')[0].split("\n")
                    access_key_id = ""
                    encrypted_secret_access_key = ""
                    for line in lines:
                        if "access_key_id" in line:
                            access_key_id = line[line.index('"') + 1:line.rindex('"')]
                        if "encrypted_secret_access_key" in line:
                            encrypted_secret_access_key = line[line.index('"') + 1:line.rindex('"')]
                            decrypted_secret_access_key = 'echo ' + '"' + encrypted_secret_access_key + '" |base64 --decode | keybase pgp decrypt'
                            decrypted_secret_access_key = os.popen(decrypted_secret_access_key).read()
                else:
                    print(self.bcolor.WARNING, "You don't have an AWS account in this project...")
                print(self.bcolor.OKCYAN, "Adding project info to .terragit file ...")
                content = self.get_file_content()
                if {"name": group_name, "group_id": group_id} not in content['projects']:

                    content['projects'].append({"name": group_name, "group_id": group_id})

                    content[group_name] = {"selected": "false", "group_id": group_id, "path": project_path,
                                           "aws_credentials": {
                                               "access_key": access_key_id,
                                               "secret_access_key": decrypted_secret_access_key
                                           }}
                    with open(os.path.expanduser(os.path.join("~/.terragit")), 'w') as f:
                        json.dump(content, f, indent=3)
                elif {"name": group_name, "group_id": group_id} in content['projects'] and content[group_name] == {
                    "selected": "false", "group_id": group_id, "path": project_path, "aws_credentials": {
                        "access_key": "",
                        "secret_access_key": ""
                    }}:
                    content[group_name] = {"selected": "false", "group_id": group_id, "path": project_path,
                                           "aws_credentials": {
                                               "access_key": access_key_id,
                                               "secret_access_key": decrypted_secret_access_key
                                           }}
                    with open(os.path.expanduser(os.path.join("~/.terragit")), 'w') as f:
                        json.dump(content, f, indent=3)
                else:
                    print("content existant dans .terragit!!")
                print(self.bcolor.OKGREEN, "Project information has been successfully added to .terragit")

        else:
            TerraConf.add_credentials(self, git_url, gitlab_token, group_id, project_id, path1, service_path, exist)

    def add_credentials(self, git_url, gitlab_token, group_id, project_id, path, service_path, exist):
        gitlab_functions1 = GitlabFunctions.GitlabFunctions(gitlab_token, git_url)
        prj_path = gitlab_functions1.get_project_by_id(project_id)['web_url']
        group_name = gitlab_functions1.get_group_by_id(group_id)['name']
        access_key_id = ""
        decrypted_secret_access_key = ""
        project_path = path
        if exist == "true":
            print(self.bcolor.OKCYAN, "Getting & Decrypting credentials ...")
            path = path + prj_path[len(git_url) + 1:len(prj_path)] + service_path
            cmd = "cd " + path + " && export gitlab_token=" + gitlab_token + " && terragrunt output"
            popen = subprocess.Popen(cmd, stdin=subprocess.PIPE, stdout=subprocess.PIPE, shell=True,
                                     encoding='utf8')
            lines = popen.communicate(input='\n')[0].split("\n")
            access_key_id = ""
            encrypted_secret_access_key = ""
            for line in lines:
                if "access_key_id" in line:
                    access_key_id = line[line.index('"') + 1:line.rindex('"')]
                if "encrypted_secret_access_key" in line:
                    encrypted_secret_access_key = line[line.index('"') + 1:line.rindex('"') - 1]
            decrypted_secret_access_key = "echo " + encrypted_secret_access_key + " | base64 --decode | keybase pgp decrypt"
            decrypted_secret_access_key = os.popen(decrypted_secret_access_key).read()
        else:
            print(self.bcolor.WARNING, "You don't have an AWS account in this project...")
        print(self.bcolor.OKCYAN, "Adding project info to .terragit file ...")
        content = self.get_file_content()
        if {"name": group_name, "group_id": group_id} not in content['projects']:
            content['projects'].append({"name": group_name, "group_id": group_id})
            content[group_name] = {"selected": "false", "group_id": group_id, "path": project_path, "aws_credentials": {
                "access_key": access_key_id,
                "secret_access_key": decrypted_secret_access_key
            }}
            with open(os.path.expanduser(os.path.join("~/.terragit")), 'w') as f:
                json.dump(content, f, indent=3)
        print(self.bcolor.OKGREEN, "Project information has been successfully added to .terragit")



import multiprocessing
import sys
import argparse
from multiprocessing.pool import ThreadPool
import terragit.terracommand as terracommande
import terragit.terradocs as terradocse
import terragit.clone as clone
import terragit.addUserGitlabRepo as add_user_gitlab_repository
import terragit.terraConf as terra_conf
import terragit.terragrunt as terragrunt
import terragit.utilsFunctions as utils
import terragit.terraClean as terraClean
import terragit.projectStatus as ProjectStatus
import terragit.gitlabFunctions as GitlabFunctions
import os


def main():

    parser = argparse.ArgumentParser()
    colors = terragrunt.bcolors()
    terraconf = terra_conf.TerraConf()

    parser.add_argument("-u", "--gitlab_url", dest="gitlab_url", default="https://gitlab.com", help="gitlab url")
    verif = terraconf.verif_file_and_credentials_existence()
    if (sys.argv[1] in ["config"]) or (verif != "false"):
        if sys.argv[1] == "config":
            terraconf.init_file()


        if sys.argv[1] == "docs":
            parser.add_argument("-m", "--module", action="store_true", help="module")
            parser.add_argument("-l", "--live", action="store_true", help="live")
            parser.add_argument("-o", "--output", dest="output_path", default="./", help="output path")
            parser.add_argument("-p", "--project_id", dest="project_id", default=None, help="id of project")
            parser.add_argument(sys.argv[1])

            args = parser.parse_args()
            terradoc = terradocse.terradoc(args.gitlab_url, verif["gitlab_token"], args.project_id)
            terradoc.docs(args.module, args.live)

        if sys.argv[1] == "aws-adduser":
            group_id = os.getenv("GROUP_ID")
            parser.add_argument("-gu", "--gitlab_username", dest="gitlab_username", default=None,
                                help="member gitlab username")
            parser.add_argument(sys.argv[1])
            args = parser.parse_args()
            add_user_aws = utils.UtilsFunctions(verif["gitlab_token"], args.gitlab_url)
            add_user_aws.add_aws_account(group_id, args.gitlab_username)

        # if sys.argv[1] == "plan":
        #
        #     parser.add_argument("-gn", "--group_name", dest="group_name", default=None,
        #                        help="your group name")
        #     parser.add_argument(sys.argv[1])
        #     args = parser.parse_args()
        #     order = terracommande.terracommand("","","","","","","","")
        #     order.terragrunt_plan(args.group_name)

        if sys.argv[1] == "clone":
            print("hello from clone command")
            parser.add_argument("-g", "--group_id", dest="group_id", default=None, help="group id")
            parser.add_argument("-p", "--project_id", dest="project_id", default=None, help="id of project")
            parser.add_argument("-ip", "--infra_project_id", dest="infra_project_id",
                                default=None, help="id of the infrastructure project")
            parser.add_argument("-path", "--path", dest="path", default="./",
                                help="path in which the project will be cloned")

            parser.add_argument(sys.argv[1])
            args = parser.parse_args()
            print(colors.OKCYAN, "Cloning ...")
            clonn = clone.Clone(verif["gitlab_token"], args.gitlab_url, args.path)
            clonn.clone_projects(args.project_id, args.group_id)
            print(colors.OKCYAN, "Checking the existance of IAM user ...")
            utilsFunctions = utils.UtilsFunctions(verif["gitlab_token"], args.gitlab_url)
            gt = GitlabFunctions.GitlabFunctions(verif["gitlab_token"], args.gitlab_url)
            service_path = utilsFunctions.check_content_existance_in_serviceGr(args.group_id,args.infra_project_id,
                            'https://git@gitlab.com/commons-acp/terraform/aws/user.git','name="' + verif["gitlab_user"] + '"')
            print("service_path",service_path)
            if "content_found" in service_path:
                print(colors.OKGREEN, "IAM user found!")

                terraconf.add_credentials_to_group(args.gitlab_url, verif["gitlab_token"], args.group_id,
                                                      args.infra_project_id, args.path, "/"+service_path[29:service_path.rindex("/")], "true")
            else:
                print("iam user not found")
                terraconf.add_credentials_to_group(args.gitlab_url, verif["gitlab_token"],args.group_id,
                                          args.infra_project_id, args.path, "", "false")

        if sys.argv[1] == "adduser":
            parser.add_argument("-g", "--group_id", dest="group_id", default=None, help="group id")
            parser.add_argument("-p", "--project_id", dest="project_id", default=None,
                                help="id of infrastructure project")
            parser.add_argument("-gu", "--gitlab_username", dest="gitlab_username", default=None,
                                help="member gitlab username")
            parser.add_argument("-lvl", "--access_level", dest="access_level", default=None,
                                help="User access level can be guest, reporter, developer, maintainer or owner")
            parser.add_argument("-k", "--keybase_username", dest="keybase_username", default=None,
                                help="user keybase username")
            parser.add_argument(sys.argv[1])
            args = parser.parse_args()
            add_user_gitlab_repo = add_user_gitlab_repository.AddUserGitlabRepo(verif["gitlab_token"], args.gitlab_url)
            exist = add_user_gitlab_repo.existance_gitlab(args.project_id, args.group_id, args.gitlab_username)
            if exist[0] == 'user exist in gitlab project & ':
                print(colors.FAIL + "user exist in gitlab project")
            else:
                add_user_gitlab_repo.add_user_repository(args.project_id, args.group_id, args.gitlab_username, args.access_level,
                                                 exist, args.keybase_username)

        if sys.argv[1] == "list":
            content = terraconf.get_file_content()
            terraconf.list_projects(content)

        if sys.argv[1] == "currentProject":
            content = terraconf.get_file_content()
            terraconf.get_selected_project(content)

        if sys.argv[1] == "switch":
            parser.add_argument("-g", "--group_id", dest="group_id", default=None, help="gitlab group id")
            parser.add_argument("-gn", "--group_name", dest="group_name", default=None, help="gitlab group name")
            parser.add_argument(sys.argv[1])
            args = parser.parse_args()
            content = terraconf.get_file_content()
            terraconf.switch_project(content, args.group_id, args.group_name)

        if sys.argv[1] == "clean":
            parser.add_argument("-g", "--group_id", dest="group_id", default=None, help="gitlab group id")
            parser.add_argument("-p", "--project_id", dest="project_id", default=None, help="gitlab project id")
            parser.add_argument("-t", "--time", dest="time", default=90, help="provide time to delete "
                                "branches/merge-requests older then time provided")
            parser.add_argument("-mr", "--mr", dest="mr", help="set it to true if you want to delete Mr else false")
            parser.add_argument("-b", "--branches", dest="branches", help="set it to true if you want to delete "
                                                                          "branches else false")
            parser.add_argument(sys.argv[1])
            args = parser.parse_args()
            terra_clean = terraClean.TerraClean(verif["gitlab_token"], args.gitlab_url)
            terra_clean.clean(args.group_id, args.project_id, int(args.time), args.mr, args.branches)

        if sys.argv[1] == "help":
            parser.add_argument(sys.argv[1])
            args = parser.parse_args()
            util = utils.UtilsFunctions(verif["gitlab_token"], args.gitlab_url)
            util.help()

        if sys.argv[1] == "status":
            parser.add_argument("-dir", "--directory", dest="directory", default=None, help="gitlab path")
            parser.add_argument(sys.argv[1])
            args = parser.parse_args()
            status = ProjectStatus.ProjectStatus(colors)
            status.gitstatus(args.directory)

    else:
        print(colors.FAIL, "terragit is not initialised please run `terragit config` command first")

    if sys.argv[1] == "plan":

        parser.add_argument("-gn", "--group_name", dest="group_name", default=None,
                            help="your group name")
        parser.add_argument(sys.argv[1])
        args = parser.parse_args()
        order = terracommande.terracommand("","","","","","","","")
        order.terragrunt_plan(args.group_name)


if __name__ == '__main__':
    main()



import json
import shutil
import subprocess
import threading
import time
import multiprocessing
from math import ceil
from multiprocessing.pool import ThreadPool
from os.path import exists


from tqdm import tqdm

from terragit.terragrunt import *
import terragit.terraConf as terra_conf

class terracommand(terragrunt, bcolors):
    def __init__(self, idProject, idCommit, idMr, gitlab_token, git_url, directory, verbose, ci_commit_title=""):
        super().__init__(verbose)
        self.idProject = idProject
        self.idCommit = idCommit
        self.idMr = idMr
        self.gitlab_token = gitlab_token
        self.git_url = git_url
        self.ci_commit_title = ci_commit_title
        self.directory = directory
        self.verbose = verbose
        self.terraconf = terra_conf.TerraConf()

    def terragruntCommand(self, command):
        mylist = []
        print("state", )
        if self.directory != None:
            mylist.append(self.directory)
        else:
            gl = gitlab.Gitlab(self.git_url, private_token=self.gitlab_token)
            project = gl.projects.get(self.idProject)
            if self.idCommit != None:
                commit = project.commits.get(self.idCommit)
                diff = commit.diff()
            if self.idMr != None:
                mr = project.mergerequests.get(self.idMr)
                diff = mr.changes()['changes']
            folderList = []
            if (len(diff) == 0):
                if not isdir(pathlib.Path(self.ci_commit_title).absolute().as_posix()):
                    print(bcolors.FAIL + self.ci_commit_title + " is not valid path" + bcolors.ENDC)
                else:
                    print("len(diff)==0 else ")
                    self.ci_commit_titlePath = pathlib.Path(self.ci_commit_title).absolute().as_posix()
                    self.pathList.append(self.ci_commit_titlePath)
                    self.printlog(command, self.pathList, self.logsFolder, self.verbose)
            else:
                for change in diff:
                    print("change ", change)
                    newPath = change['new_path']
                    if not ("live/") in newPath:
                        print(pathlib.Path(
                            newPath).absolute().as_posix() + bcolors.WARNING + " OUT of SCOPE" + bcolors.ENDC)
                    else:
                        pathh = pathlib.Path(newPath).parent.absolute().as_posix()
                        folderList.append(pathh)

            mylist = list(dict.fromkeys(folderList))
            print("mylist ", mylist)

        for path in mylist:
            print("mylist for ", mylist)
            if (isdir(path)):
                print("is dir ")
                self.getAllFolder(path)
                if command == "changes":
                    print(mylist)
                    return mylist
        self.printlog(command, self.pathList, self.logsFolder, self.verbose)
        if self.failedloglist:
            if self.verbose:
                for message in self.failedloglist:
                    logfileName = message.split("live/")[1].replace("/", "_")
                    os.chdir(self.failedlogsFolder)

                    shutil.move(self.logsFolder + "/" + logfileName + ".log", "failed_" + logfileName + ".log")
            sys.exit(1)

    def terragrunt_plan(self, group_name):

        token_ci_id = os.getenv("CI_JOB_TOKEN")
        path = os.getenv("my_path")
        list = []
        path_absolute = ""
        # if token_ci_id is None:
        #     path = self.terraconf.get_file_content()
        #     for root, dirs, files in os.walk(path[group_name]['path']):
        #         list.append(root)
        #     i = 0
        #     while str(group_name)+"-grp/infra" not in list[i]:
        #         i += 1
        #         if str(group_name)+"-grp/infra" in list[i]:
        #             path_absolute = list[i]
        #             break
        if token_ci_id is not None:
            path_absolute =path
            print("path_absolute", path_absolute)
        subdir = []
        for root, subdirs, files in os.walk(path_absolute):
            if os.path.lexists(os.path.join(root, "terragrunt.hcl")):
                subdir.append(root)
                print("subdir", subdir)
        cwd = ['/builds/clients-acp/202201/chosa-grp/infra/live/git/users/hiba',
               '/builds/clients-acp/202201/chosa-grp/infra/live/git/users/hiba.jaouadi',
               '/builds/clients-acp/202201/chosa-grp/infra/live/git/users/kacem.yedes']
        number_of_task = len(cwd)
        progress_bar = tqdm(total=number_of_task)
        process = 'terragrunt plan -out=tfplan && terragrunt show -json tfplan > tfplan.json'
        processes = []
        for c in cwd:
            popen = subprocess.Popen(process, stdout=subprocess.PIPE, shell=True, encoding='utf8', cwd=c)
            processes.append(popen)
        for p in processes:
            p.wait()
            progress_bar.update()
        for c in cwd:
            exit = exists(os.path.expanduser(os.path.join(c+"/tfplan.json")))
            print(exit)
        # processes = []
        # processes1 = []
        # cwd = [path+'/live/aws/global/iam/users/hiba',
        #        path+'/live/aws/global/iam/users/hiba.jaouadi',
        #        path+'/live/aws/global/iam/users/kacem.yedes']
        # number_of_task = len(cwd)
        # progress_bar = tqdm(total=number_of_task)
        # # progress_bar2 = tqdm(total=number_of_task)
        # print(bcolors.OKGREEN, "collecting information ..")
        # process = 'source ~/my_profile && terragrunt plan -out=tfplan'
        # process1 = 'source ~/my_profile ; terragrunt show -json tfplan > tfplan.json'
        # process2 = 'source ~/my_profile && terragrunt plan -out=tfplan && terragrunt show -json tfplan > tfplan.json'
        # for c in cwd:
        #     if "webapp" not in c:
        #         proc = subprocess.Popen(["/bin/bash", "-c", process], bufsize=8192, stdin=None, stdout=subprocess.PIPE, stderr=subprocess.PIPE, cwd=c)
        #         processes.append(proc)
        # for p in processes:
        #     p.wait()
        #     progress_bar.update()
        # for c in cwd:
        #     if "webapp" not in c:
        #         proc1 = subprocess.Popen(["/bin/bash", "-c", process1], bufsize=8192, stdin=None, stdout=subprocess.PIPE ,stderr=subprocess.PIPE, cwd=c)
        #         processes1.append(proc1)
        # for p in processes1:
        #     p.wait()
        # for c in cwd:
        #     if "webapp" in c:
        #         proc = subprocess.run(["/bin/bash", "-c", process2],shell=False,bufsize=8192, stdout=subprocess.PIPE,stdin=None, stderr=subprocess.PIPE, cwd=c)
        #         progress_bar.update()
        #
        # files = []
        # for i in range(len(cwd)):
        #     if exists(os.path.expanduser(os.path.join(cwd[i], "tfplan.json"))):
        #         file = open(os.path.expanduser(os.path.join(cwd[i], "tfplan.json")))
        #         data = file.readlines()
        #         print("myyyy data",data)
        #         json_object = json.loads(data[2])
        #         file.close()
        #         files.append(json_object['resource_changes'])
        #
        # added = 0
        # changed = 0
        # deleted = 0
        #
        # for i in range(len(files)):
        #     print(bcolors.ENDC, "You re working in folder" , bcolors.OKBLUE, cwd[i])
        #     for j in files[i]:
        #         if j['change']['actions'] == ['create']:
        #             added += 1
        #             print(bcolors.OKGREEN, "+", bcolors.ENDC, ' ressource', j['address'], 'will be added')
        #         if j['change']['actions'] == ['update']:
        #             changed += 1
        #             print(bcolors.WARNING,"~", bcolors.ENDC, ' ressource', j['address'], 'will be updated in-place')
        #         if j['change']['actions'] == ['delete']:
        #             changed += 1
        #             print(bcolors.FAIL ,"-", bcolors.ENDC, ' ressource', j['address'], 'will be destroyed')
        # print("Plan:", bcolors.OKGREEN, added, bcolors.ENDC,"to add, ", bcolors.WARNING,changed, bcolors.ENDC," to change, ",bcolors.FAIL,deleted,bcolors.ENDC," to destroy.")



import subprocess

import requests # token : gitlab_token
import os

class Compare:
    def __init__(self):

        #,tf_state1, tf_state2
        def list_state(self, infra):
            cmd = " cd " + infra + " && terragrunt state list < tf"
            popen = subprocess.Popen(cmd, stdin=subprocess.PIPE, stdout=subprocess.PIPE, shell=True,
                                     encoding='utf8')
            lines = popen.communicate(input='\n')[0].split("\n")


import requests # token : gitlab_token
import os

class Clone:
    def __init__(self,gitlab_token,git_url,root_path):
        self.root_path =root_path
        self.gitlab_token= gitlab_token
        self.git_url=git_url
    def clone_projects(self,project_id,grp_id):
        headers = {'PRIVATE-TOKEN': self.gitlab_token}
        root_path=self.root_path
        if project_id !=None:
            print("project id not null")
            url_projets= self.git_url+"/api/v4/projects/"+str(project_id)
            project  = requests.get(url_projets, headers=headers).json()
            path = root_path+"/"+project['path_with_namespace']
            if not os.path.isdir(path):
                os.makedirs(path)
                cmd = "git clone "+project['ssh_url_to_repo'] +" "+path
                os.system(cmd)
            else:
                print(f"{path} already exists")

        if grp_id !=None:

            url_projets = self.git_url+"/api/v4/groups/"+str(grp_id)+"/projects"

            projets = requests.get(url_projets, headers=headers).json()
            for project in projets:

                path = root_path+project['path_with_namespace']
                if not os.path.isdir(path):
                    os.makedirs(path)
                    cmd = "git clone "+project['ssh_url_to_repo'] +" "+path
                    os.system(cmd)
                else:
                    print(f"{path} already exists")

            url_grps = self.git_url+"/api/v4/groups/"+str(grp_id)+"/subgroups"
            subgroups = requests.get(url_grps, headers=headers).json()
            for subgroup in subgroups :
                self.clone_projects(None,subgroup['id'])









from terragit.gitlabFunctions import *
from datetime import datetime, timedelta
import terragit.terragrunt as terragrunt
from math import *



class TerraClean:
    def __init__(self, gitlab_token, git_url, ):
        self.bcolor = terragrunt.bcolors
        self.gitlab_functions = GitlabFunctions(gitlab_token, git_url)

    def clean(self, group_id, project_id, time, mr, branches):
        if project_id is not None:
            if branches == 'true':
                self.get_project_branches(project_id, time)
                return
            elif mr:
                self.clean_mrs(project_id, group_id, time)
                return
        elif group_id is not None:
            if branches == 'true':
                projects = []
                branches_to_delete = []
                page = 1
                per_page = 40
                while per_page == 40:
                    projects = projects + self.gitlab_functions.get_projects_of_group(group_id, page, per_page)
                    page = page + 1
                    per_page = len(projects)
                for project in projects:
                    branches_to_delete = branches_to_delete + self.get_project_branches(project['id'], time, "grp")
                self.delete_branches(branches_to_delete)
            elif mr == 'true':
                self.clean_mrs(project_id, group_id, time)

    def get_project_branches(self, project_id, time, param=""):
        branches = self.gitlab_functions.get_all_project_branches(project_id)

        branches_to_delete = []
        for br in branches:
            if 'commit' in br:
                commit_date = br['commit']['committed_date']
                commit_date = commit_date[0:commit_date.index('T')]
                datetime_object = datetime.strptime(commit_date, '%Y-%m-%d')
                commit_time_in_days = (datetime.utcnow() - datetime_object).days
                merged = ""
                if (commit_time_in_days >= time) and not (br["protected"]) and (br["merged"]):
                    print(
                        self.bcolor.WARNING + "branch " + br["name"] + " is outdated by " + str(commit_time_in_days) +
                        " and it has been merged but not deleted its web_url is: " + br["web_url"])
                    br["project_id"] = project_id
                    branches_to_delete.append(br)
                elif (commit_time_in_days >= time) and not (br["protected"]) and not br["merged"]:
                    print(self.bcolor.WARNING + "last commit in branch " + br["name"] +
                          " it has been created " + str(commit_time_in_days) + " days ago, its web_url is: " + br[
                              "web_url"])
                    br["project_id"] = project_id
                    branches_to_delete.append(br)
                elif (commit_time_in_days >= time) and (br["protected"]) and br["merged"]:
                    br["project_id"] = project_id
                    branches_to_delete.append(br)
                    print(self.bcolor.WARNING + "branch " + br["name"] +
                          " has been merged but not deleted its web_url is: " + br["web_url"])
        if param == "grp":
            return branches_to_delete
        else:
            self.delete_branches(branches_to_delete)

    def delete_branches(self, branches_to_delete):
        if len(branches_to_delete) == 0:
            print(self.bcolor.OKGREEN, "project is clean")
            return
        value = input(self.bcolor.OKBLUE + "would you like to delete all these branches at once, delete "
                                           "one by one or cancel clean operation ?[all/oneByOne/cancel]")
        if value == "all":
            for br in branches_to_delete:
                self.gitlab_functions.delete_branch(br["project_id"], br["name"])
        elif value == "oneByOne":
            for br in branches_to_delete:
                if input(self.bcolor.OKBLUE + "would you like to delete ths branch " +
                         br["name"] + " ?[yes/no]") == "yes":
                    self.gitlab_functions.delete_branch(br["project_id"], br["name"])

    def clean_mrs(self, project_id, group_id, time):
        created_before = datetime.now() - timedelta(days=time)
        created_before = created_before.strftime("%Y-%m-%dT%H:%M:%SZ")
        mrs_to_delete = []

        if project_id is not None:
            first_mrs = len(self.gitlab_functions.get_number_merge_request_project(project_id, created_before, 20, 1))

            if first_mrs == 20:
                count = first_mrs
                i = 1
                somme = 20
                while count % 20 == 0:
                    i += 1
                    other_mrs_count_intermidate= len(self.gitlab_functions.get_number_merge_request_project(project_id, created_before, 20, i))
                    somme += other_mrs_count_intermidate
                    if other_mrs_count_intermidate != 20:
                        break
            else:
                somme = 5

            a = ceil(somme / 20)
            for i in range(1, a+1):
                mrs_to_delete.insert(i-1, self.gitlab_functions.get_all_merge_request_by_page(project_id, created_before, i))

        elif group_id is not None:
            first_mrs = len(self.gitlab_functions.get_number_merge_request_group(group_id, created_before, 20, 1))

            if first_mrs == 20:
                count = first_mrs
                i = 1
                somme = 20
                while count % 20 == 0:
                    i += 1
                    other_mrs_count_intermidate = len(self.gitlab_functions.get_number_merge_request_group(group_id , created_before, 20, i))
                    somme += other_mrs_count_intermidate
                    if other_mrs_count_intermidate != 20:
                        break
            else:
                somme = 5
            a = ceil(somme / 20)
            for i in range(1, a+1):
                mrs_to_delete.insert(i-1, self.gitlab_functions.get_all_group_merge_requests_per_page(group_id, created_before, i))

        if len(mrs_to_delete[0]) == 0:
            print(self.bcolor.OKGREEN, "project is clean")

        else:

            for i in range(0, len(mrs_to_delete)):
                for j in range(0, len(mrs_to_delete[i])):
                    print(self.bcolor.WARNING + "Merge request " + mrs_to_delete[i][j]["title"] + " is created before " + str(
                        created_before))
                value = input(self.bcolor.OKBLUE + "would you like to delete all these merge requests at once, delete "
                                                    "one by one or cancel clean operation ?[all/oneByOne/cancel]")

        if value == "all":
            for i in range(0, len(mrs_to_delete)):
                for j in range(0, len(mrs_to_delete[i])):
                    self.gitlab_functions.update_mr_state(mrs_to_delete[i][j]["project_id"], mrs_to_delete[i][j]["iid"])
        elif value == "oneByOne":
            for i in range(0, len(mrs_to_delete)):
                for j in range(0, len(mrs_to_delete[i])):
                    if input(self.bcolor.OKBLUE + "would you like to delete this merge requests " +
                             mrs_to_delete[i][j]["title"] + " ?[yes/no]") == "yes":
                        self.gitlab_functions.update_mr_state(mrs_to_delete[i][j]["project_id"], mrs_to_delete[i][j]["iid"])

        elif value == "cancel":
            return
        print(self.bcolor.OKGREEN, "project has been cleaned successfully")
